#ifndef included_SIDL_DLL_IOR
#define included_SIDL_DLL_IOR

#ifndef NULL
#define NULL 0
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#ifdef __cplusplus
extern "C" {
#endif

/* Symbol SIDL.DLL */

/* These structures are defined in this header */
struct SIDL_DLL__object;

/* These functions are defined in the implementation */
extern struct SIDL_DLL__object *SIDL_DLL__new(void);
extern void SIDL_DLL__init(struct SIDL_DLL__object *self);
extern void SIDL_DLL__fini(struct SIDL_DLL__object *self);

/* Forward declarations for array structure and external classes and interfaces */
struct SIDL_DLL__array;
struct SIDL_Loader__object;
struct SIDL_Loader__array;
struct SIDL_BaseClass__object;
struct SIDL_BaseClass__array;
struct SIDL_BaseInterface__object;
struct SIDL_BaseInterface__array;

#ifndef included_SIDL_header_h
#include "SIDL_header.h"
#endif
//
// Includes for all interface and class dependencies
//
#ifndef included_SIDL_BaseClass_IOR
#include "SIDL/BaseClass_IOR/BaseClass_IOR.h"
#endif

/* Declare the method entry point vector */
struct SIDL_DLL__epv {
  /* Builtin methods */
  void *(*f__cast_ior)(
    struct SIDL_DLL__object *self,
    const char *name);
  void (*f__delete)(
    struct SIDL_DLL__object *self);
  void (*f__ctor)(
    struct SIDL_DLL__object *self);
  void (*f__dtor)(
    struct SIDL_DLL__object *self);
  /* Methods contributed by SIDL.DLL */
  SIDL_bool (*f_loadLibrary)(
    struct SIDL_DLL__object *self,
    /*in*/ const char * uri);
  char * (*f_getLibraryName)(
    struct SIDL_DLL__object *self);
  void (*f_unloadLibrary)(
    struct SIDL_DLL__object *self);
  void * (*f_lookupSymbol)(
    struct SIDL_DLL__object *self,
    /*in*/ const char * linker_name);
  struct SIDL_BaseClass__object * (*f_createClass)(
    struct SIDL_DLL__object *self,
    /*in*/ const char * sidl_name);
  /* Methods contributed by SIDL.BaseClass */
  /* Methods contributed by SIDL.BaseInterface */
  void (*f_addReference)(
    struct SIDL_DLL__object *self);
  void (*f_deleteReference)(
    struct SIDL_DLL__object *self);
  SIDL_bool (*f_isSame)(
    struct SIDL_DLL__object *self,
    /*in*/ struct SIDL_BaseInterface__object * iobj);
  struct SIDL_BaseInterface__object * (*f_queryInterface)(
    struct SIDL_DLL__object *self,
    /*in*/ const char * name);
  SIDL_bool (*f_isInstanceOf)(
    struct SIDL_DLL__object *self,
    /*in*/ const char * name);
};

/* Define the object structure */
struct SIDL_DLL__object {
  struct SIDL_BaseClass__object d_sidl_baseclass;
  struct SIDL_DLL__epv * d_epv;
  void *d_object;
  void *d_data;
};

/* Create a dense array of the given dimension with specified index bounds */
/*   This array owns and manages its data */
/*   All object pointers are initialized to NULL */
struct SIDL_DLL__array * SIDL_DLL__iorarray_create(
  int32_t dimen,
  const int32_t lower[],
  const int32_t upper[]);

/* Create an array using user-provided memory */
struct SIDL_DLL__array * SIDL_DLL__iorarray_borrow(
  struct SIDL_DLL__object ** firstElement,
  int32_t dimen,
  const int32_t lower[],
  const int32_t upper[],
  const int32_t stride[]);

/* Destroy the array (NULL arrays are ignored) */
void SIDL_DLL__iorarray_destroy(
  struct SIDL_DLL__array *array);

/* Return the number of dimensions in the array */
/*   If the array pointer is NULL, zero is returned */
int32_t SIDL_DLL__iorarray_dimen(
  const struct SIDL_DLL__array *array);

/* Return the lower bound on dimension ind */
/*   If the array pointer is NULL or ind is not a valid dimension, zero is returned */
int32_t SIDL_DLL__iorarray_lower(
  const struct SIDL_DLL__array *array,
  int32_t ind);

/* Return the upper bound on dimension ind */
/*   If the array pointer is NULL or ind is not a valid dimension, -1 is returned */
int32_t SIDL_DLL__iorarray_upper(
  const struct SIDL_DLL__array *array,
  int32_t ind);

/* Get an element of a multi-dimensional array */
/* This will use the indices provided up to the actual dimension of the array, ignoring extra indices */
/* If the return value is non-NULL, the client owns one reference to the object/interface */
/* The client must decrement the reference count when done with the reference */
struct SIDL_DLL__object * SIDL_DLL__iorarray_get(
  const struct SIDL_DLL__array *array,
  const int32_t indices[]);

/* get an element of a multi-dimensional array */
/* This will use the indices provided up to the actual dimension of the array, ignoring extra indices */
/* If the return value is non-NULL, the client owns one reference to the object/interface */
/* The client must decrement the reference count when done with the reference */
struct SIDL_DLL__object * SIDL_DLL__iorarray_get4(
  const struct SIDL_DLL__array *array,
  int32_t i1,
  int32_t i2,
  int32_t i3,
  int32_t i4);

/* Set an element of a multi-dimensional array */
/* This will use the indices provided up to the actual dimension of the array, ignoring extra indices */
/* If the incoming value is non-NULL, this function will increment the reference code of the object/interface */
/* If it is overwriting a non-NULL pointer, the reference count of the object/interface being overwritten will be decremented */
void SIDL_DLL__iorarray_set(
  struct SIDL_DLL__array *array,
  const int32_t indices[],
  struct SIDL_DLL__object * value);

/* set an element of a multi-dimensional array */
/* This will use the indices provided up to the actual dimension of the array, ignoring extra indices */
/* If the incoming value is non-NULL, this function will increment the reference code of the object/interface */
/* If it is overwriting a non-NULL pointer, the reference count of the object/interface being overwritten will be decremented */
void SIDL_DLL__iorarray_set4(
  struct SIDL_DLL__array *array,
  int32_t i1,
  int32_t i2,
  int32_t i3,
  int32_t i4,
  struct SIDL_DLL__object * value);

struct SIDL_DLL__external {
  struct SIDL_DLL__object *(*createObject)(void);
  void (*remoteDummy)(void);
  struct SIDL_DLL__array * (*createArray)  (
    int32_t dimen,
    const int32_t lower[],
    const int32_t upper[]);
  struct SIDL_DLL__array * (*borrowArray)  (
    struct SIDL_DLL__object ** firstElement,
    int32_t dimen,
    const int32_t lower[],
    const int32_t upper[],
    const int32_t stride[]);
  void (*destroyArray)  (
    struct SIDL_DLL__array *array);
  int32_t (*getDimen)  (
    const struct SIDL_DLL__array *array);
  int32_t (*getLower)  (
    const struct SIDL_DLL__array *array,
    int32_t ind);
  int32_t (*getUpper)  (
    const struct SIDL_DLL__array *array,
    int32_t ind);
  struct SIDL_DLL__object * (*getElement)  (
    const struct SIDL_DLL__array *array,
    const int32_t indices[]);
  struct SIDL_DLL__object * (*getElement4)  (
    const struct SIDL_DLL__array *array,
    int32_t i1,
    int32_t i2,
    int32_t i3,
    int32_t i4);
  void (*setElement)  (
    struct SIDL_DLL__array *array,
    const int32_t indices[],
    struct SIDL_DLL__object * value);
  void (*setElement4)  (
    struct SIDL_DLL__array *array,
    int32_t i1,
    int32_t i2,
    int32_t i3,
    int32_t i4,
    struct SIDL_DLL__object * value);
};

/* This function returns a pointer to a static structure of pointers to function entry points */
/* It is called upon loading the DLL */
const struct SIDL_DLL__external *SIDL_DLL__externals(void);
#ifdef __cplusplus
}
#endif

#endif /* included_SIDL_DLL_IOR */
