import config.base

class Configure(config.base.Configure):
  def __init__(self, framework):
    config.base.Configure.__init__(self, framework)
    functions = []
    headersC  = ['dlfcn.h', 'inttypes.h', 'sys/dl.h']
    libraries = [('dl', 'dlopen')]
    # Needed by ltdl.cc
    functions += ['bcopy', 'dlerror', 'index', 'memcpy', 'rindex', 'shl_load', 'strchr', 'strcmp', 'strrchr']
    headersC  += map(lambda name: name+'.h', ['ctype', 'dl', 'malloc', 'memory', 'stdio', 'stdlib', 'string', 'strings'])
    # Need a test for NEED_USCORE
    # Punted on all F77 stuff
    # Add dependencies
    self.compilers = self.framework.require('config.compilers', self)
    self.types     = self.framework.require('config.types',     self)
    self.headers   = self.framework.require('config.headers',   self)
    self.functions = self.framework.require('config.functions', self)
    self.libraries = self.framework.require('config.libraries', self)
    self.headers.headers.extend(headersC)
    self.functions.functions.extend(functions)
    self.libraries.libraries.extend(libraries)
    return

  def checkDL(self):
    '''Check that dynamic library functionality is available'''
    if self.libraries.haveLib('dl'):
      self.framework.addSubstitution('RUNTIME_EXTRA_LIB', 'libdl.so')
    elif not self.functions.haveFunction('dlerror'):
      raise RuntimeError('Must have dynamic library capability in order to use the SIDL Runtime')
    else:
      self.framework.addSubstitution('RUNTIME_EXTRA_LIB', '')
    return

  def configurePython(self):
    '''Check whether Python uses a shared library'''
    import distutils.sysconfig
    import os

    lib = os.path.join(distutils.sysconfig.get_config_var('LIBPL'), distutils.sysconfig.get_config_var('LDLIBRARY'))
    ext = distutils.sysconfig.get_config_var('SO')
    # if .so was not built then need to strip .a off of end
    if lib[-2:] == '.a': lib = lib[0:-2]
    # may be stuff after .so like .0, so cannot use splitext()
    lib = lib.split(ext)[0]+ext
    if os.path.isfile(lib):
      self.framework.addDefine('PYTHON_SHARED_LIBRARY', '"'+lib+'"')
    return

  def configureLTDL(self):
    '''Mess around with stupid GNU library which we should throw away'''
    self.framework.addDefine('LTDL_OBJDIR', '".libs"')
    return

  def configure(self):
    import os

    self.framework.headerBottom = '''
/* Set flags for dynamic or static loading of implementations in stubs.
 * One and only one of SIDL_STATIC_LIBRARY and SIDL_DYNAMIC_LIBRARY may be set.
 * If neither is set, then SIDL_DYNAMIC_LIBRARY is chosen as the default.
 */
#if (!defined(SIDL_STATIC_LIBRARY) && !defined(SIDL_DYNAMIC_LIBRARY))
#define SIDL_DYNAMIC_LIBRARY
#endif

#if (defined(SIDL_STATIC_LIBRARY) && defined(SIDL_DYNAMIC_LIBRARY))
#error Cannot define both SIDL_STATIC_LIBRARY and SIDL_DYNAMIC_LIBRARY
#endif
#if (!defined(SIDL_STATIC_LIBRARY) && !defined(SIDL_DYNAMIC_LIBRARY))
#error Must define one of SIDL_STATIC_LIBRARY or SIDL_DYNAMIC_LIBRARY
#endif

/*
 * The USE_DL_IMPORT flag is required for proper Python linking under CYGWIN.
 */
#if defined(__CYGWIN__) && !defined(USE_DL_IMPORT)
#define USE_DL_IMPORT
#endif'''
    self.executeTest(self.checkDL)
    self.executeTest(self.configurePython)
    self.executeTest(self.configureLTDL)
    return
