#!/usr/bin/env python
import user
import build.framework
import project

import os

class PetscMake(build.framework.Framework):
  def __init__(self, clArgs = None, argDB = None):
    build.framework.Framework.__init__(self, project.Project('bk://sidl.bkbits.net/Runtime', self.getRoot()), clArgs, argDB)
    self.project.setWebDirectory('petsc@terra.mcs.anl.gov://mcs/www-unix/ase')
    return

  def t_configure(self):
    self.configureHeader = os.path.join(self.project.getRoot(), self.sidlTemplate.usingSIDL.getServerRootDir('Cxx', 'sidl'), 'sidl_config.h')
    return build.framework.Framework.t_configure(self)

  def defineDirectories(self):
    self.directories['sidl']    = os.path.join(self.project.getRoot(), 'sidl')
    self.directories['driver']  = os.path.join(self.project.getRoot(), 'driver')
    self.directories['bin']     = os.path.join(self.project.getRoot(), 'bin')
    return

  def defineSIDL(self):
    '''Define SIDL files sets before activation since they are necessary to calculate the compiler template'''
    import build.fileset
    self.filesets['sidl']  = build.fileset.ExtensionFileSet(self.directories['sidl'], '.sidl')
    self.filesets['etags'] = build.fileset.FileSet(['TAGS'], mustExist = 0)
    return

  def defineSource(self):
    '''Define rooted files sets here since they require project activation'''
    import build.fileset
    serverRoot = self.sidlTemplate.usingSIDL.getServerRootDir('Cxx', 'sidl')
    pythonRoot = self.sidlTemplate.usingSIDL.getClientRootDir('Python')
    url        = self.project.getUrl()

    self.filesets['core']   = build.fileset.RootedFileSet(url, [os.path.join(serverRoot, 'ltdl.cc'), os.path.join(serverRoot, 'ltdl.hh')], tag = 'cxx server sidl')
    self.filesets['ior']    = build.fileset.RootedFileSet(url, map(lambda d: os.path.join(serverRoot, 'SIDL', d, d+'.c'),
                                                                   ['bool_IOR', 'char_IOR', 'dcomplex_IOR', 'double_IOR', 'fcomplex_IOR', 'float_IOR', 'int_IOR',
                                                                    'long_IOR', 'opaque_IOR', 'string_IOR']), tag = 'cxx server sidl')
    self.filesets['python'] = build.fileset.RootedFileSet(url, map(lambda f: os.path.join(pythonRoot, f+'.c'), ['SIDLObjA', 'SIDLPyArrays', 'OpaqueObject_Module']),
                                                          tag = 'python client')
    self.filesets['sidl'].children.extend([self.filesets['core'], self.filesets['ior'], self.filesets['python']])
    return

  def setupProject(self):
    '''This is called before project activation'''
    self.defineDirectories()
    self.defineSIDL()
    return

  def setupBuild(self):
    '''This is called after project activation, but before the build'''
    self.defineSource()

    # Bootstrap problem: SIDL.Loader cannot separate its client and IOR (since externals cannot be dynamically loaded)
    self.sidlTemplate.addStaticPackage('sidl')
    self.sidlTemplate.addServer('Cxx')

    # Need dlopen(), etc.
    self.compileTemplate.extraLibraries.append(project.ArgumentPath('RUNTIME_EXTRA_LIB'))
    return

if __name__ ==  '__main__':
  import sys
  PetscMake(sys.argv[1:]).main()
