#ifndef included_SIDL_BaseClass_IOR
#include "SIDL/BaseClass_IOR/BaseClass_IOR.h"
#endif

/* Static variables for managing EPV initialization */
static int s_method_initialized = 0;

static struct SIDL_BaseClass__epv s_new__sidl_baseclass;

static struct SIDL_BaseInterface__epv s_new__sidl_baseinterface;

/* Declare EPV routines defined in the skeleton */
extern void SIDL_BaseClass__set_epv(struct SIDL_BaseClass__epv * epv);

/* CAST: dynamic type casting support */
static void *SIDL_BaseClass__cast_ior(
  struct SIDL_BaseClass__object *self,
  const char *name)
{
  void *cast = NULL;
  /* We are guaranteed here that the first member in any IOR is a BaseInterface object */
  struct SIDL_BaseClass__object *s0 = ((struct SIDL_BaseInterface__object *) self)->d_object;

  if (!strcmp(name, "SIDL.BaseClass")) {
    cast = (void *) s0;
  } else if (!strcmp(name, "SIDL.BaseInterface")) {
    cast = (void *) &s0->d_sidl_baseinterface;
  }

  return cast;
}

/* DELETE: call destructor and free object memory */
static void SIDL_BaseClass__delete(
  struct SIDL_BaseClass__object *self)
{
  struct SIDL_BaseClass__object * s0 = self->d_epv->f__cast_ior(self, "SIDL.BaseClass");
  SIDL_BaseClass__fini(s0);
  memset((void *) s0, 0, sizeof(struct SIDL_BaseClass__object));
  free((void *) s0);
}

/* INIT_EPV: create method entry point vector (EPV) structure */
static void SIDL_BaseClass__init_epv(
  struct SIDL_BaseClass__object *self)
{
  struct SIDL_BaseClass__object *s0 = self;

  struct SIDL_BaseClass__epv * epv = &s_new__sidl_baseclass;
  struct SIDL_BaseInterface__epv * e0 = &s_new__sidl_baseinterface;

  epv->f__cast_ior = SIDL_BaseClass__cast_ior;
  epv->f__delete = SIDL_BaseClass__delete;
  epv->f__ctor = NULL;
  epv->f__dtor = NULL;
  epv->f_addReference = NULL;
  epv->f_deleteReference = NULL;
  epv->f_isSame = NULL;
  epv->f_queryInterface = NULL;
  epv->f_isInstanceOf = NULL;

  SIDL_BaseClass__set_epv(epv);

  e0->f__cast_ior = (void *(*)(void *, const char *)) epv->f__cast_ior;
  e0->f__delete = (void (*)(void *)) epv->f__delete;
  e0->f_addReference = (void (*)(void *)) epv->f_addReference;
  e0->f_deleteReference = (void (*)(void *)) epv->f_deleteReference;
  e0->f_isSame = (SIDL_bool (*)(void *, struct SIDL_BaseInterface__object *)) epv->f_isSame;
  e0->f_queryInterface = (struct SIDL_BaseInterface__object * (*)(void *, const char *)) epv->f_queryInterface;
  e0->f_isInstanceOf = (SIDL_bool (*)(void *, const char *)) epv->f_isInstanceOf;

  s_method_initialized = 1;
}

/* NEW: allocate object and initialize it */
struct SIDL_BaseClass__object *SIDL_BaseClass__new()
{
  struct SIDL_BaseClass__object *self = (struct SIDL_BaseClass__object *) malloc(sizeof(struct SIDL_BaseClass__object));
  SIDL_BaseClass__init(self);
  return self;
}

/* INIT: initialize a new instance of the class object */
void SIDL_BaseClass__init(
  struct SIDL_BaseClass__object *self)
{
  struct SIDL_BaseClass__object *s0 = self;

  if (!s_method_initialized) {
    SIDL_BaseClass__init_epv(s0);
  }

  /* Setup d_epv members */
  s0->d_sidl_baseinterface.d_epv = &s_new__sidl_baseinterface;
  s0->d_epv = &s_new__sidl_baseclass;

  /* Setup d_object members */
  s0->d_sidl_baseinterface.d_object = self;
  s0->d_object = self;

  /* Setup d_data members */
  s0->d_data = NULL;
  (*(self->d_epv->f__ctor))(self);
}

/* FINI: deallocate a class instance (destructor) */
void SIDL_BaseClass__fini(
  struct SIDL_BaseClass__object *self)
{
  struct SIDL_BaseClass__object *s0 = self;

  (*(self->d_epv->f__dtor))(self);

}

/* The IOR array structure */
/*   Macros to read this are defined in SIDLArray.h */
struct SIDL_BaseClass__array {
  struct SIDL_BaseClass__object ** d_firstElement;
  int32_t *d_lower;
  int32_t *d_upper;
  int32_t *d_stride;
  SIDL_bool d_borrowed;
  int32_t d_dimen;
};

static struct SIDL_BaseClass__array *newArray(int32_t dimen, const int32_t lower[], const int32_t upper[]) {
  static const size_t arraySize = sizeof(struct SIDL_BaseClass__array) + (sizeof(int32_t) - (sizeof(struct SIDL_BaseClass__array)%sizeof(int32_t)))%sizeof(int32_t);
  struct SIDL_BaseClass__array *result = (struct SIDL_BaseClass__array *) malloc(arraySize + 3*sizeof(int32_t)*dimen);

  result->d_dimen    = dimen;
  result->d_borrowed = 0;
  result->d_lower    = (int32_t *) ((char *) result + arraySize);
  result->d_upper    = result->d_lower + dimen;
  result->d_stride   = result->d_upper + dimen;
  memcpy(result->d_lower, lower, sizeof(int32_t)*dimen);
  memcpy(result->d_upper, upper, sizeof(int32_t)*dimen);
  return result;
}

/* Create a dense array of the given dimension with specified index bounds */
/*   This array owns and manages its data */
/*   All object pointers are initialized to NULL */
struct SIDL_BaseClass__array * SIDL_BaseClass__iorarray_create(
  int32_t dimen,
  const int32_t lower[],
  const int32_t upper[])
{
  struct SIDL_BaseClass__array *result = newArray(dimen, lower, upper);
  int32_t size;
  int32_t i;

  for(i = 0, size = 1; i < dimen; i++) {
    result->d_stride[i] = size;
    size *= (upper[i] - lower[i] + 1);
  }
  size *= sizeof(struct SIDL_BaseClass__object *);
  result->d_firstElement = (struct SIDL_BaseClass__object **) malloc((size_t) size);
  memset(result->d_firstElement, 0, (size_t) size);
  return result;
}

/* Create an array using user-provided memory */
struct SIDL_BaseClass__array * SIDL_BaseClass__iorarray_borrow(
  struct SIDL_BaseClass__object ** firstElement,
  int32_t dimen,
  const int32_t lower[],
  const int32_t upper[],
  const int32_t stride[])
{
  struct SIDL_BaseClass__array *result = newArray(dimen, lower, upper);

  memcpy(result->d_stride, stride, sizeof(int32_t)*dimen);
  result->d_firstElement = firstElement;
  result->d_borrowed     = 1;
  return result;
}

/* Destroy the array (NULL arrays are ignored) */
void SIDL_BaseClass__iorarray_destroy(
  struct SIDL_BaseClass__array *array)
{
  if (array) {
    const int32_t dimen = array->d_dimen;

    if (!(array->d_borrowed)) {
      if (dimen > 0) {
        struct SIDL_BaseClass__object **start = array->d_firstElement;
        struct SIDL_BaseClass__object **end;
        int32_t size = 1;

        if (dimen > 1) {
          size = array->d_stride[dimen-1];
        }
        size *= (array->d_upper[dimen-1] - array->d_lower[dimen-1] + 1);
        end   = start + size;
        while(start < end) {
          if (*start) {
            (*(((struct SIDL_BaseClass__object *) (*start)->d_object)->d_epv->f_deleteReference))((*start)->d_object);
            *start = NULL;
          }
          ++start;
        }
      }
      free(array->d_firstElement);
    }
    memset(array, 0, sizeof(struct SIDL_BaseClass__array) + 3*sizeof(int32_t)*dimen);
    free(array);
  }
}

/* Return the number of dimensions in the array */
/*   If the array pointer is NULL, zero is returned */
int32_t SIDL_BaseClass__iorarray_dimen(
  const struct SIDL_BaseClass__array *array)
{
  return array ? array->d_dimen : 0;
}

/* Return the lower bound on dimension ind */
/*   If the array pointer is NULL or ind is not a valid dimension, zero is returned */
int32_t SIDL_BaseClass__iorarray_lower(
  const struct SIDL_BaseClass__array *array,
  int32_t ind)
{
  return (array && (ind < array->d_dimen) && (ind >= 0)) ? array->d_lower[ind] : 0;
}

/* Return the upper bound on dimension ind */
/*   If the array pointer is NULL or ind is not a valid dimension, -1 is returned */
int32_t SIDL_BaseClass__iorarray_upper(
  const struct SIDL_BaseClass__array *array,
  int32_t ind)
{
  return (array && (ind < array->d_dimen) && (ind >= 0)) ? array->d_upper[ind] : -1;
}

/* Get an element of a multi-dimensional array */
/* This will use the indices provided up to the actual dimension of the array, ignoring extra indices */
/* If the return value is non-NULL, the client owns one reference to the object/interface */
/* The client must decrement the reference count when done with the reference */
struct SIDL_BaseClass__object * SIDL_BaseClass__iorarray_get(
  const struct SIDL_BaseClass__array *array,
  const int32_t indices[])
{
  struct SIDL_BaseClass__object **result = NULL;
  if (array && (array->d_dimen > 0)) {
    int32_t i;

    result = array->d_firstElement;
    for(i = 0; i < array->d_dimen; i++) {
      if ((indices[i] >= array->d_lower[i]) && (indices[i] <= array->d_upper[i])) {
        result += (array->d_stride[i]*(indices[i] - array->d_lower[i]));
      } else {
        result = NULL;
        break;
      }
    }
  }
  if (result) {
    if (*result) {
      (*(((struct SIDL_BaseClass__object *) (*result)->d_object)->d_epv->f_addReference))((*result)->d_object);
    }
    return *result;
  } else {
    return NULL;
  }
}

/* get an element of a multi-dimensional array */
/* This will use the indices provided up to the actual dimension of the array, ignoring extra indices */
/* If the return value is non-NULL, the client owns one reference to the object/interface */
/* The client must decrement the reference count when done with the reference */
struct SIDL_BaseClass__object * SIDL_BaseClass__iorarray_get4(
  const struct SIDL_BaseClass__array *array,
  int32_t i1,
  int32_t i2,
  int32_t i3,
  int32_t i4)
{
  struct SIDL_BaseClass__object **result = NULL;
  if (array) {
    result = array->d_firstElement;
    switch(array->d_dimen) {
    case 4:
      if ((i4 >= array->d_lower[3]) && (i4 <= array->d_upper[3])) {
        result += (array->d_stride[3]*(i4 - array->d_lower[3]));
      } else {
        result = NULL;
        break;
      }
      /* fall through */
    case 3:
      if ((i3 >= array->d_lower[2]) && (i3 <= array->d_upper[2])) {
        result += (array->d_stride[2]*(i3 - array->d_lower[2]));
      } else {
        result = NULL;
        break;
      }
      /* fall through */
    case 2:
      if ((i2 >= array->d_lower[1]) && (i2 <= array->d_upper[1])) {
        result += (array->d_stride[1]*(i2 - array->d_lower[1]));
      } else {
        result = NULL;
        break;
      }
      /* fall through */
    case 1:
      if ((i1 >= array->d_lower[0]) && (i1 <= array->d_upper[0])) {
        result += (array->d_stride[0]*(i1 - array->d_lower[0]));
      } else {
        result = NULL;
        break;
      }
      break;
    default:
      result = NULL;
      break;
    }
  }
  if (result) {
    if (*result) {
      (*(((struct SIDL_BaseClass__object *) (*result)->d_object)->d_epv->f_addReference))((*result)->d_object);
    }
    return *result;
  } else {
    return NULL;
  }
}

/* Set an element of a multi-dimensional array */
/* This will use the indices provided up to the actual dimension of the array, ignoring extra indices */
/* If the incoming value is non-NULL, this function will increment the reference code of the object/interface */
/* If it is overwriting a non-NULL pointer, the reference count of the object/interface being overwritten will be decremented */
void SIDL_BaseClass__iorarray_set(
  struct SIDL_BaseClass__array *array,
  const int32_t indices[],
  struct SIDL_BaseClass__object * value)
{
  struct SIDL_BaseClass__object **result = NULL;
  if (array && (array->d_dimen > 0)) {
    int32_t i;

    result = array->d_firstElement;
    for(i = 0; i < array->d_dimen; i++) {
      if ((indices[i] >= array->d_lower[i]) && (indices[i] <= array->d_upper[i])) {
        result += (array->d_stride[i]*(indices[i] - array->d_lower[i]));
      } else {
        result = NULL;
        break;
      }
    }
    if (result) {
      if (value) {
        (*(((struct SIDL_BaseClass__object *) value->d_object)->d_epv->f_addReference))(value->d_object);
      }
      if (*result) {
        (*(((struct SIDL_BaseClass__object *) (*result)->d_object)->d_epv->f_deleteReference))((*result)->d_object);
      }
      *result = value;
    }
  }
}

/* set an element of a multi-dimensional array */
/* This will use the indices provided up to the actual dimension of the array, ignoring extra indices */
/* If the incoming value is non-NULL, this function will increment the reference code of the object/interface */
/* If it is overwriting a non-NULL pointer, the reference count of the object/interface being overwritten will be decremented */
void SIDL_BaseClass__iorarray_set4(
  struct SIDL_BaseClass__array *array,
  int32_t i1,
  int32_t i2,
  int32_t i3,
  int32_t i4,
  struct SIDL_BaseClass__object * value)
{
  struct SIDL_BaseClass__object **result = NULL;
  if (array) {
    result = array->d_firstElement;
    switch(array->d_dimen) {
    case 4:
      if ((i4 >= array->d_lower[3]) && (i4 <= array->d_upper[3])) {
        result += (array->d_stride[3]*(i4 - array->d_lower[3]));
      } else {
        result = NULL;
        break;
      }
      /* fall through */
    case 3:
      if ((i3 >= array->d_lower[2]) && (i3 <= array->d_upper[2])) {
        result += (array->d_stride[2]*(i3 - array->d_lower[2]));
      } else {
        result = NULL;
        break;
      }
      /* fall through */
    case 2:
      if ((i2 >= array->d_lower[1]) && (i2 <= array->d_upper[1])) {
        result += (array->d_stride[1]*(i2 - array->d_lower[1]));
      } else {
        result = NULL;
        break;
      }
      /* fall through */
    case 1:
      if ((i1 >= array->d_lower[0]) && (i1 <= array->d_upper[0])) {
        result += (array->d_stride[0]*(i1 - array->d_lower[0]));
      } else {
        result = NULL;
        break;
      }
      break;
    default:
      result = NULL;
      break;
    }
  }
  if (result) {
    if (value) {
      (*(((struct SIDL_BaseClass__object *) value->d_object)->d_epv->f_addReference))(value->d_object);
    }
    if (*result) {
      (*(((struct SIDL_BaseClass__object *) (*result)->d_object)->d_epv->f_deleteReference))((*result)->d_object);
    }
    *result = value;
  }
}

static const struct SIDL_BaseClass__external s_externalEntryPoints = {
  SIDL_BaseClass__new,
  NULL /*SIDL_BaseClass__remote*/,
  SIDL_BaseClass__iorarray_create,
  SIDL_BaseClass__iorarray_borrow,
  SIDL_BaseClass__iorarray_destroy,
  SIDL_BaseClass__iorarray_dimen,
  SIDL_BaseClass__iorarray_lower,
  SIDL_BaseClass__iorarray_upper,
  SIDL_BaseClass__iorarray_get,
  SIDL_BaseClass__iorarray_get4,
  SIDL_BaseClass__iorarray_set,
  SIDL_BaseClass__iorarray_set4
};

/* This function returns a pointer to a static structure of pointers to function entry points */
/* It is called upon loading the DLL */
const struct SIDL_BaseClass__external *SIDL_BaseClass__externals()
{
  return &s_externalEntryPoints;
}
