#ifndef included_SIDL_cxx_hh
#include "SIDL_cxx.hh"
#endif

#ifndef included_SIDL_DLL_IOR
#include "SIDL/DLL_IOR/DLL_IOR.h"
#endif

#ifndef included_SIDL_DLL_impl
#define included_SIDL_DLL_impl

//
// Includes for all method dependencies
//
#ifndef included_SIDL_Loader_stub
#include "SIDL/Loader/Loader.hh"
#endif
#ifndef included_SIDL_BaseClass_stub
#include "SIDL/BaseClass/BaseClass.hh"
#endif
#ifndef included_SIDL_BaseInterface_stub
#include "SIDL/BaseInterface/BaseInterface.hh"
#endif
#ifndef included_SIDL_DLL_stub
#include "SIDL/DLL/DLL.hh"
#endif

// DO-NOT-DELETE splicer.begin(SIDL.DLL._includes)
#include <dlfcn.h>
#ifndef RTLD_LOCAL
#define RTLD_LOCAL 0
#endif
#ifdef HAVE_LTDL
#ifndef LTDL_H
#include "ltdl.h"
#endif
#endif
#ifdef HAVE_LIBWWW
#include "wwwconf.h"
#include "WWWLib.h"
#include "WWWInit.h"
#endif
// DO-NOT-DELETE splicer.end(SIDL.DLL._includes)

namespace SIDL {
  // Symbol "SIDL.DLL"
  class DLL_impl {
  private:
    // Pointer back to IOR
    // Use this to dispatch back through IOR vtable
    DLL self;

    // DO-NOT-DELETE splicer.begin(SIDL.DLL._implementation)
#ifdef HAVE_LTDL
    lt_dlhandle libraryHandle;
#else
    void       *libraryHandle;
#endif
    std::string libraryName;

#ifdef HAVE_LIBWWW
    char *url_to_local_file(const char *url);
#endif
#ifdef HAVE_LTDL
    void check_lt_initialized(void);
#endif
    // DO-NOT-DELETE splicer.end(SIDL.DLL._implementation)

  // Constructors/Destructors 
  private:
    // private default constructor (required)
    DLL_impl() {}

  public:
    // SIDL constructor (required)
    // Note: alternate Skel constructor does not call addref()
    // (fixes bug #275)
    DLL_impl(struct SIDL_DLL__object *s) : self(s, true) {_ctor();}

    // virtual destructor (required)
    virtual ~DLL_impl() {_dtor();}

  // User Constructors/Destructors 
  public:
    void _ctor();
    void _dtor();

  // User defined methods 
  public:
    bool
    SIDL::DLL_impl::loadLibrary(
      /*in*/ std::string uri)
    throw ()
    ;
    
    std::string
    SIDL::DLL_impl::getLibraryName()
    throw ()
    ;
    
    void
    SIDL::DLL_impl::unloadLibrary()
    throw ()
    ;
    
    void*
    SIDL::DLL_impl::lookupSymbol(
      /*in*/ std::string linker_name)
    throw ()
    ;
    
    ::SIDL::BaseClass
    SIDL::DLL_impl::createClass(
      /*in*/ std::string sidl_name)
    throw ()
    ;
    
  }; // end class DLL_impl
} // end namespace SIDL

// DO-NOT-DELETE splicer.begin(SIDL.DLL._misc)
#ifdef HAVE_LTDL
// The libtool dynamic loading library must be initialized before any calls
// are made to library routines.  Initialize only once.
void SIDL::DLL_impl::check_lt_initialized(void) {
  static bool initialized = false;

  if (!initialized) {
    (void) lt_dlinit();
    initialized = true;
  }
}
#endif

#ifdef HAVE_LIBWWW
std::string SIDL::DLL_impl::url_to_local_file(std::string url) {
  HTRequest   *request = NULL;
  std::string tmpfile;

#ifdef __CYGWIN__
  tmpfile  = "cyg";
  tmpfile += tmpnam(NULL);
  tmpfile += ".dll";
#else
  tmpfile  = "lib";
  tmpfile += tmpnam(NULL);
  tmpfile += ".so";
#endif

  HTProfile_newPreemptiveClient("SIDL_DLL", "1.0");
  HTAlert_setInteractive(0);

  request = HTRequest_new();
  HTLoadToFile(url, request, tmpfile.c_str());
  HTRequest_delete(request);
  HTProfile_delete();

  return tmpfile;
}
#endif
// DO-NOT-DELETE splicer.end(SIDL.DLL._misc)

#endif /*included_SIDL_DLL_impl*/
