#ifndef included_SIDL_cxx_hh
#include "SIDL_cxx.hh"
#endif

#ifndef included_SIDL_Loader_IOR
#include "SIDL/Loader_IOR/Loader_IOR.h"
#endif

#ifndef included_SIDL_Loader_impl
#define included_SIDL_Loader_impl

//
// Includes for all method dependencies
//
#ifndef included_SIDL_DLL_stub
#include "SIDL/DLL/DLL.hh"
#endif
#ifndef included_SIDL_BaseClass_stub
#include "SIDL/BaseClass/BaseClass.hh"
#endif
#ifndef included_SIDL_BaseInterface_stub
#include "SIDL/BaseInterface/BaseInterface.hh"
#endif
#ifndef included_SIDL_Loader_stub
#include "SIDL/Loader/Loader.hh"
#endif

// DO-NOT-DELETE splicer.begin(SIDL.Loader._includes)
#include <map>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>

typedef struct DLL_List {
  struct DLL_List *next;
  SIDL::DLL        dll;
} DLL_List;
// DO-NOT-DELETE splicer.end(SIDL.Loader._includes)

namespace SIDL {
  // Symbol "SIDL.Loader"
  class Loader_impl {
  private:
    // Pointer back to IOR
    // Use this to dispatch back through IOR vtable
    Loader self;

    // DO-NOT-DELETE splicer.begin(SIDL.Loader._implementation)
    // Defined in addDLL
    static DLL_List   *dllList;
    // Defined in addSearchPath
    static std::string searchPath;
    // Defined in createClass: Holds associations between class names and implementing libraries
    static std::map<std::string, std::string> dllMap;

    // Initialize the search path if it has not yet been initialized.  The initial
    // search path value is taken from environment variables SIDL_DLL_PATH and PYTHONPATH.
    static void initialize_search_path(void) {
      static bool searchInit = false;

      if (!searchInit) {
        std::string sidlPath;
        std::string pythonPath;
        char       *path;

        path = getenv("SIDL_DLL_PATH");
        if (path) {sidlPath   = path;}
        path = getenv("PYTHONPATH");
        if (path) {pythonPath = path;}

        SIDL::Loader_impl::searchPath += sidlPath;
        if (!sidlPath.empty() && !pythonPath.empty()) {
          SIDL::Loader_impl::searchPath += ":";
        }
        SIDL::Loader_impl::searchPath += pythonPath;

        FILE *fd = popen("python -c 'import user; import sidldllpath; print sidldllpath.getSIDLDLLPath()'", "r");
        if (fd) {
	  char buffer[10240];

	  fgets(buffer, 10240, fd);
          if (buffer[0] == '/') {
	    for (int i = 0; i < 10240; i++){
	      if (buffer[i] == '\n') {
                buffer[i] = '\0';
                break;
              }
	    }
	    SIDL::Loader_impl::searchPath += ";";
	    SIDL::Loader_impl::searchPath += buffer;
          }
	  pclose(fd);
        }

        fd = popen("python -c 'import user; import sidldllpath; print sidldllpath.getSIDLDLLMap()'", "r");
        if (fd) {
	  if (getc(fd) == '{') {
            int ch;

            // Read an entry
            while ((ch = getc(fd)) != '}') {
              std::string cls, lib;

              // Read class name
              if (ch != '\'') {
                // Invalid structure
              }
              while ((ch = getc(fd)) != '\'') {
                cls += ch;
              }
              // Read separator
              if (getc(fd) != ':') {
                // Invalid structure
              }
              if (getc(fd) != ' ') {
                // Invalid structure
              }
              // Read library
              if (getc(fd) != '\'') {
                // Invalid structure
              }
              while ((ch = getc(fd)) != '\'') {
                lib += ch;
              }
              // Add entry
              SIDL::Loader_impl::dllMap[cls] = lib;
              // Read separator
              ch = getc(fd);
              if (ch == '}') break;
              if (ch != ',') {
                // Invalid structure
              }
              if (getc(fd) != ' ') {
                // Invalid structure
              }
            }
          }
	  pclose(fd);
        }
        searchInit = true;
      }
    }

    // Initialize the list of DLLs if it has not yet been initialized.  The initial
    // DLL list contains only the single DLL library "main:".
    static void initialize_dll_list(void) {
      static bool initialized = false;

      if (!initialized) {
        SIDL::DLL dll = SIDL::DLL::_create();

        if (dll.loadLibrary("main:")) {
          DLL_List *item = new DLL_List;

          item->dll  = dll;
          item->next = NULL;
          SIDL::Loader_impl::dllList = item;
        } else {
          dll.deleteReference();
        }
        initialized = true;
      }
    }

    static bool is_directory(std::string file) {
      struct stat status;

      return ((!stat(file.c_str(), &status)) && (S_ISDIR(status.st_mode)));
    }

    static std::string util_getcwd(void) {
      std::string cwd  = NULL;
      size_t      size = 64;

      while(true) {
        cwd.reserve(size+1);

        if (getcwd((char *) cwd.data(), size) != NULL) {
          break;
        } else if (errno == ERANGE) {
          size *= 2;
        } else {
          break;
        }
      }

      return cwd;
    }

    // Return whether the specified URI refers to a file.  We assume that any
    // URI that we do not understand points to a file.
    static bool is_file_uri(std::string uri) {
      bool isFile = true;

      if (uri.substr(0, 5) == "file:") {
        isFile = true;
      } else if (uri.substr(0, 5) == "main:") {
        isFile = false;
      } else if (uri.substr(0, 4) == "lib:") {
        isFile = false;
#ifdef HAVE_LIBWWW
      } else if (uri.substr(0, 4) == "ftp:") {
        isFile = false;
      } else if (uri.substr(0, 5) == "http:") {
        isFile = false;
#endif
      }

      return isFile;
    }

    // Search through the list of DLLs to find out whether we have already
    // loaded the specified uri.
    static bool already_loaded(std::string uri) {
      DLL_List *head = NULL;
      bool      same = false;

      SIDL::Loader_impl::initialize_dll_list();

      head = SIDL::Loader_impl::dllList;
      while((head != NULL) && !same) {
        same = uri == head->dll.getLibraryName();
        head = head->next;
      }

      return same;
    }

    // Try to load the DLL corresponding to the specified URI and look up
    // the specified name.  If the name cannot be found, then return null.
    static void *try_load_dll(std::string uri, std::string linker_name) {
      void *symbol = NULL;

      if (!SIDL::Loader_impl::already_loaded(uri)) {
        SIDL::DLL dll = SIDL::DLL::_create();

        if (dll.loadLibrary(uri)) {
          SIDL::Loader_impl::addDLL(dll);
          symbol = dll.lookupSymbol(linker_name);
        }
        dll.deleteReference();
      }

      return symbol;
    }

    // Try to find the specified symbol in the specified file URI.  The URI
    // must begin with the "file:" protocol.  If the file is a directory, then
    // recursively search all possible dll files and subdirectories for the
    // match.
    static void *find_symbol(std::string uri, std::string linker_name) {
      void *symbol = NULL;

      if (uri.substr(0, 5) == "file:") {
        std::string file = uri.substr(5);

        if (!SIDL::Loader_impl::is_directory(file)) {
          // If not a directory, then open the file and check if dll opens OK.
          // If we support LTDL, then allow .la extensions.  If on CYGWIN,
          // then also allow .dll extensions.  Otherwise, allow .so extensions.
#ifdef HAVE_LTDL
          if (uri.substr(uri.length()-3, 3) == ".la") {
            symbol = SIDL::Loader_impl::try_load_dll(uri, linker_name);
          }
#endif
#ifdef __CYGWIN__
          if (uri.substr(uri.length()-4, 4) == ".dll") {
            symbol = SIDL::Loader_impl::try_load_dll(uri, linker_name);
          }
#else
          if (uri.substr(uri.length()-3, 3) == ".so") {
            symbol = SIDL::Loader_impl::try_load_dll(uri, linker_name);
          }
#endif
        } else {
          // If file is a directory, then recurse through the directory
          DIR *dir = opendir(file.c_str());

          if (dir) {
            struct dirent *entry = NULL;

            while ((entry = readdir(dir)) != NULL) {
              if (entry->d_name[0] != '.') {
                std::string f = "file:"+file+ "/";

                f     += entry->d_name;
                symbol = SIDL::Loader_impl::find_symbol(f, linker_name);
                if (symbol) break;
              }
            }
            closedir(dir);
          }
        }
      }

      return symbol;
    }
    // DO-NOT-DELETE splicer.end(SIDL.Loader._implementation)

  // Constructors/Destructors 
  private:
    // private default constructor (required)
    Loader_impl() {}

  public:
    // SIDL constructor (required)
    // Note: alternate Skel constructor does not call addref()
    // (fixes bug #275)
    Loader_impl(struct SIDL_Loader__object *s) : self(s, true) {_ctor();}

    // virtual destructor (required)
    virtual ~Loader_impl() {_dtor();}

  // User Constructors/Destructors 
  public:
    void _ctor();
    void _dtor();

  // User defined methods 
  public:
    static void
    SIDL::Loader_impl::setSearchPath(
      /*in*/ std::string path_name)
    throw ()
    ;
    
    static std::string
    SIDL::Loader_impl::getSearchPath()
    throw ()
    ;
    
    static void
    SIDL::Loader_impl::addSearchPath(
      /*in*/ std::string path_fragment)
    throw ()
    ;
    
    static bool
    SIDL::Loader_impl::loadLibrary(
      /*in*/ std::string uri)
    throw ()
    ;
    
    static void
    SIDL::Loader_impl::addDLL(
      /*in*/ ::SIDL::DLL dll)
    throw ()
    ;
    
    static void
    SIDL::Loader_impl::unloadLibraries()
    throw ()
    ;
    
    static void*
    SIDL::Loader_impl::lookupSymbol(
      /*in*/ std::string linker_name)
    throw ()
    ;
    
    static ::SIDL::BaseClass
    SIDL::Loader_impl::createClass(
      /*in*/ std::string sidl_name)
    throw ()
    ;
    
  }; // end class Loader_impl
} // end namespace SIDL

// DO-NOT-DELETE splicer.begin(SIDL.Loader._misc)
// DO-NOT-DELETE splicer.end(SIDL.Loader._misc)

#endif /*included_SIDL_Loader_impl*/
