/*
 * File:        SIDL_bool_IOR.h
 * Copyright:   (c) 2001 The Regents of the University of California
 * Release:     $Name: release-0-6-1 $
 * Revision:    @(#) $Revision: 1.7 $
 * Date:        $Date: 2001/07/11 22:20:51 $
 * Description: Bool array declarations and definitions
 *
 */

#ifndef included_SIDL_bool_IOR_h
#define included_SIDL_bool_IOR_h

#ifndef included_SIDLType_h
#include "SIDLType.h"
#endif

/**
 * The data structure for multi-dimensional arrays for SIDL bools.
 * The client may access this with the functions below or using
 * the macros in the header file SIDLArray.h.
 */
struct SIDL_bool__array {
  SIDL_bool     *d_firstElement;
  int32_t       *d_lower;
  int32_t       *d_upper;
  int32_t       *d_stride;
  int32_t        d_dimen;
  SIDL_bool      d_borrowed;
};

#ifdef __cplusplus
extern "C" { /*}*/
#endif

/**
 * Create a dense bool array of the given dimension with specified
 * index bounds.  This array owns and manages its data. This function
 * does not initialize the contents of the array.
 */
struct SIDL_bool__array*
SIDL_bool__array_create(int32_t       dimen,
                        const int32_t lower[],
                        const int32_t upper[]);

/**
 * Create an array that uses data memory from another source. This initial
 * contents are determined by the data being borrowed.
 */
struct SIDL_bool__array*
SIDL_bool__array_borrow(SIDL_bool* firstElement,
                        int32_t        dimen,
                        const int32_t  lower[],
                        const int32_t  upper[],
                        const int32_t  stride[]);

/**
 * Destroy the given array. Trying to destroy a NULL array is a
 * noop.
 */
void
SIDL_bool__array_destroy(struct SIDL_bool__array* array);

#define SIDL_bool__array_get1(a, i1) \
  SIDL_bool__array_get4((a), (i1), 0, 0, 0)

#define SIDL_bool__array_get2(a, i1, i2) \
  SIDL_bool__array_get4((a), (i1), (i2), 0, 0)

#define SIDL_bool__array_get3(a, i1, i2, i3) \
  SIDL_bool__array_get4((a), (i1), (i2), (i3), 0)

/**
 * Get an element of a multi-dimensional array. This will use
 * the indices provided up to the actual dimension of the array.
 * The values of excess indices are ignored.
 */
SIDL_bool
SIDL_bool__array_get4(const struct SIDL_bool__array* array,
                      int32_t                            i1,
                      int32_t                            i2,
                      int32_t                            i3,
                      int32_t                            i4);

/**
 * Get an element of a multi-dimensional array. This will use
 * the indices provided up to the actual dimension of the array.
 * The values of excess indices are ignored.
 */
SIDL_bool
SIDL_bool__array_get(const struct SIDL_bool__array* array,
                     const int32_t                      indices[]);

#define SIDL_bool__array_set1(a, i1, value) \
  SIDL_bool__array_set4((a), (i1), 0, 0, 0, (value))

#define SIDL_bool__array_set2(a, i1, i2, value) \
  SIDL_bool__array_set4((a), (i1), (i2), 0, 0, (value))

#define SIDL_bool__array_set3(a, i1, i2, i3, value) \
  SIDL_bool__array_set4((a), (i1), (i2), (i3), 0, (value))

/**
 * Set an element of a multi-dimensional array. This will use
 * the indices provided up to the actual dimension of the array.
 * The values of excess indices are ignored.
 */
void
SIDL_bool__array_set4(struct SIDL_bool__array* array,
                      int32_t                      i1,
                      int32_t                      i2,
                      int32_t                      i3,
                      int32_t                      i4,
                      SIDL_bool                value);

/**
 * Set an element of a multi-dimensional array. This will use
 * the indices provided up to the actual dimension of the array.
 * The values of excess indices are ignored.
 */
void
SIDL_bool__array_set(struct SIDL_bool__array* array,
                     const int32_t                indices[],
                     SIDL_bool                value);

/**
 * Return the number of dimensions in the array. If the
 * array pointer is NULL, zero is returned.
 */
int32_t
SIDL_bool__array_dimen(const struct SIDL_bool__array *array);

/**
 * Return the lower bound on dimension ind. If ind is not
 * a valid dimension, zero is returned.
 */
int32_t
SIDL_bool__array_lower(const struct SIDL_bool__array *array, int32_t ind);

/**
 * Return the upper bound on dimension ind. If ind is not
 * a valid dimension, negative one is returned.
 */
int32_t
SIDL_bool__array_upper(const struct SIDL_bool__array *array, int32_t ind);

#ifdef __cplusplus
}
#endif
#endif /* included_SIDL_bool_IOR */
