#ifndef included_SIDL_cxx_hh
#define included_SIDL_cxx_hh

#ifndef __cplusplus
#error C++ headerfile: not meant for C compilers
#endif

// get configuration information
#ifndef INCLUDED_SIDL_CONFIG_H
#include "sidl_config.h"
#endif

// need exception
#include <exception>

// include complex<T>
#ifndef HAVE_COMPLEX
#  error C++ bindings assume complex<T> is available
#else
#  include <complex>
#endif

// include C++ strings
#include <string>

// include output streams for arrays
#include <ostream>

#if 0
// include SIDL_BaseClass__object
#ifndef included_SIDL_BaseClass_IOR
#include "SIDL/BaseClass_IOR/BaseClass_IOR.h"
#endif
#endif
#ifndef included_SIDL_header_h
#include "SIDL_header.h"
#endif

// introduce StubBase into the SIDL namespace
// StubBase is the abstract base class for all C++ stubs.
namespace SIDL { 

  // redefine types inside SIDL namespace
  // typedef bool bool;
  // typedef char char;
  // typedef ::int32_t int;
  // typedef ::int64_t long;
  // typedef float float;
  // typedef double double;
  typedef std::complex<float> fcomplex;
  typedef std::complex<double> dcomplex;
  typedef std::string string;
  typedef void * opaque;

  // This is the base class for all C++ Stubs
  class StubBase {
  public:
    virtual void* _cast(const char* type) const = 0;
    virtual ~StubBase() {}
  };

  // This is the root template that handles 
  // the guts of everything for SIDL Arrays
  template< typename array_ior_t, 
    typename item_ior_t, 
    typename item_cxx_wrapper_t > 
  class array_mixin { 
  protected:
    array_ior_t * d_array;
  public:
    // constructors
    array_mixin( array_ior_t* src ) : d_array(src) { 
    }
    array_mixin() : d_array(0) { 
    }
    // destructor
    ~array_mixin() {}

    // create
    bool create(int32_t dimen, const int32_t lower[], const int32_t upper[]) {
      d_array = (*(item_cxx_wrapper_t::_get_ext()->createArray))(dimen, lower, upper);
      return d_array != 0;
    }
    
    // borrow
    bool borrow( item_ior_t * first_element, int32_t dimen,
      const int32_t lower[], const int32_t upper[], const int32_t stride[]) {
      d_array = (*(item_cxx_wrapper_t::_get_ext()->borrowArray))(first_element, dimen, lower,
          upper, stride);
      return d_array != 0;
    }

    // destroy
    void destroy() {
      (*(item_cxx_wrapper_t::_get_ext()->destroyArray))(d_array);
    }

    // get dimension
    int32_t dim() const { 
      return (*(item_cxx_wrapper_t::_get_ext()->getDimen))(d_array);
    }
    
    // get lower bound along particular dimension
    int32_t lower( int32_t d ) const {
      return (*(item_cxx_wrapper_t::_get_ext()->getLower))(d_array,d);
    } 
    
    // get upper bound along particular dimension
    int32_t upper( int32_t d ) const {
      return (*(item_cxx_wrapper_t::_get_ext()->getUpper))(d_array,d);
    }
    
    // get stride along particular dimension
    int32_t stride( int32_t d ) const {
      return (*(item_cxx_wrapper_t::_get_ext()->getStride))(d_array,d);
    }
    
    // get a const pointer to the actual array ior 
    const array_ior_t* _get_ior() const { return d_array; }

    // get a non-const pointer to the actual array ior
    array_ior_t* _get_ior() { return d_array;}

    // check if no pointer to array
    bool _is_nil() const {
      return (d_array==0);
    }

    // check if no pointer to array
    bool _not_nil() const {
      return (d_array!=0);
    }

    // check if no pointer to array
    bool operator !() const {
      return (d_array==0);
    }

    // set the actual array ior
    void _set_ior( array_ior_t * s) { d_array=s; }

    // get
    item_cxx_wrapper_t get(int32_t i, int32_t j=0, int32_t k=0, int32_t l=0) {
      return item_cxx_wrapper_t( (*(item_cxx_wrapper_t::_get_ext()->getElement4))(d_array,i,j,k,l) );
    }

    // set
    void set(item_cxx_wrapper_t element, int32_t i, int32_t j=0, int32_t k=0, int32_t l=0) {
      return (*(item_cxx_wrapper_t::_get_ext()->setElement4))(d_array,i,j,k,l,element._get_ior());
    }
    
  };
  
  // This is a bogus template that doesn't do much.
  // But we specialize off of this to the more complicated
  // forms above.
  template< typename T > 
  class array : public array_mixin< SIDL_int__array, int32_t, int32_t > { 
  };

  // Now the template classes for built-in types.
  template<> 
  class array<bool> :  public array_mixin< struct SIDL_bool__array, SIDL_bool, bool > { 
  public:
    // default constructor 
    array() : array_mixin< struct SIDL_bool__array, SIDL_bool, bool >() {} 

    // constructor/casting operator
    array( SIDL_bool__array* src ) : array_mixin< struct SIDL_bool__array, SIDL_bool, bool >(src) {}

    // create
    bool create(int32_t dimen, const int32_t lower[], const int32_t upper[]) {
      d_array = SIDL_bool__array_create(dimen, lower, upper);
      return d_array != 0;
    }
    
    bool create(int32_t m, bool *args) {
      int32_t lower = 0,upper = m-1;
      d_array = SIDL_bool__array_create(1, &lower, &upper);
      if (!d_array) return false;
      int i; for (i=0; i<m; i++) this->set(args[i],i);
      return true;
    }


    // borrow
    bool borrow( SIDL_bool * first_element, int32_t dimen,
      const int32_t lower[], const int32_t upper[], const int32_t stride[]) {
      d_array = SIDL_bool__array_borrow(first_element, dimen, lower,
					upper, stride);
      return d_array != 0;
    }

    // destroy
    void destroy() {
      SIDL_bool__array_destroy(d_array);
    }

    // get dimension
    int32_t dim() const { 
      return SIDL_bool__array_dimen(d_array);
    }
    
    // get lower bound along particular dimension
    int32_t lower( int32_t d ) const {
      return SIDL_bool__array_lower(d_array,d);
    } 
    
    // get upper bound along particular dimension
    int32_t upper( int32_t d ) const {
      return SIDL_bool__array_upper(d_array,d);
    }
    
    // get stride along particular dimension
    // int stride( int d ) const {
    //   return SIDL_bool__array_stride(d_array,d);
    // }

    // get
    bool get(int32_t i, int32_t j=0, int32_t k=0, int32_t l=0) {
      return SIDL_bool__array_get4(d_array,i,j,k,l) == TRUE;
    }

    bool operator [](int32_t i) {return get(i);}

    friend std::ostream& operator <<(std::ostream &os, array<bool> me) {
      for (int i=me.lower(0); i<me.upper(0)+1;i++) {
	os << me.get(i) << " ";
      }
      return os;
    }

    // set
    void set(bool element, int32_t i, int32_t j=0, int32_t k=0, int32_t l=0) {
      SIDL_bool int_element = (element) ? TRUE : FALSE;
      SIDL_bool__array_set4(d_array,i,j,k,l,int_element);
    } 
  };

  template<> 
  class array<char> : public array_mixin<struct SIDL_char__array, char, char > {
  public:
    // default constructor
    array() : array_mixin< struct SIDL_char__array, char, char>() {}

    // constructor/casting operator
    array( SIDL_char__array* src ) : array_mixin< struct SIDL_char__array, char, char>(src) {}

    // create
    bool create(int32_t dimen, const int32_t lower[], const int32_t upper[]) {
      d_array = SIDL_char__array_create(dimen, lower, upper);
      return d_array != 0;
    }
    
    bool create(int32_t m, char *args) {
      int32_t lower = 0,upper = m-1;
      d_array = SIDL_char__array_create(1, &lower, &upper);
      if (!d_array) return false;
      int i; for (i=0; i<m; i++) this->set(args[i],i);
      return true;
    }

    // borrow
    bool borrow( char * first_element, int32_t dimen,
      const int32_t lower[], const int32_t upper[], const int32_t stride[]) {
      d_array = SIDL_char__array_borrow(first_element, dimen, lower,
        upper, stride);
      return d_array != 0;
    }

    // destroy
    void destroy() {
      SIDL_char__array_destroy(d_array);
    }

    // get dimension
    int32_t dim() const { 
      return SIDL_char__array_dimen(d_array);
    }
    
    // get lower bound along particular dimension
    int32_t lower( int32_t d ) const {
      return SIDL_char__array_lower(d_array,d);
    } 
    
    // get upper bound along particular dimension
    int32_t upper( int32_t d ) const {
      return SIDL_char__array_upper(d_array,d);
    }
    
    // get stride along particular dimension
    // int32_t stride( int32_t d ) const {
    // 	 return SIDL_char__array_stride(d_array,d);
    // }

    // get
    char get(int32_t i, int32_t j=0, int32_t k=0, int32_t l=0) {
      return SIDL_char__array_get4(d_array,i,j,k,l);
    }

    char operator [](int32_t i) {return get(i);}

    friend std::ostream& operator <<(std::ostream &os,array<char> me) {
      for (int i=me.lower(0); i<me.upper(0)+1;i++) {
	os << me.get(i) << " ";
      }
      return os;
    }

    // set
    void set(char element, int32_t i, int32_t j=0, int32_t k=0, int32_t l=0) {
      SIDL_char__array_set4(d_array,i,j,k,l,element);
    }
  };

  template<> 
  class array<int> : public array_mixin<struct SIDL_int__array, int32_t, int32_t > {
  public:
    // default constructor
    array() : array_mixin< struct SIDL_int__array, int32_t, int32_t >() {} 

    // constructor/casting operator
    array( SIDL_int__array* src ) : array_mixin< struct SIDL_int__array, int32_t, int32_t >(src) {} 
    
    // create
    bool create(int32_t dimen, const int32_t lower[], const int32_t upper[]) {
      d_array = SIDL_int__array_create(dimen, lower, upper);
      return d_array != 0;
    }
    
    bool create(int32_t m, int *args) {
      int32_t lower = 0,upper = m-1;
      d_array = SIDL_int__array_create(1, &lower, &upper);
      if (!d_array) return false;
      int i; for (i=0; i<m; i++) this->set(args[i],i);
      return true;
    }

    bool create(int32_t m) {
      int32_t lower = 0,upper = m-1;
      d_array = SIDL_int__array_create(1, &lower, &upper);
      return d_array != 0;
    }

    // borrow
    bool borrow( int32_t * first_element, int32_t dimen,
      const int32_t lower[], const int32_t upper[], const int32_t stride[]) {
      d_array = SIDL_int__array_borrow(first_element, dimen, lower,
        upper, stride);
      return d_array != 0;
    }

    // destroy
    void destroy() {
      SIDL_int__array_destroy(d_array);
    }

    // get dimension
    int32_t dim() const { 
      return SIDL_int__array_dimen(d_array);
    }
    
    // get lower bound along particular dimension
    int32_t lower( int32_t d ) const {
      return SIDL_int__array_lower(d_array,d);
    } 
    
    // get upper bound along particular dimension
    int32_t upper( int32_t d ) const {
      return SIDL_int__array_upper(d_array,d);
    }
    
    // get stride along particular dimension
    // int32_t stride( int32_t d ) const {
    //   return SIDL_int__array_stride(d_array,d);
    // }

    // get
    int32_t get(int32_t i, int32_t j=0, int32_t k=0, int32_t l=0) {
      return SIDL_int__array_get4(d_array,i,j,k,l);
    }

    int32_t operator [](int32_t i) {return get(i);}

    friend std::ostream& operator <<(std::ostream &os,array<int> me) {
      for (int i=me.lower(0); i<me.upper(0)+1;i++) {
	os << me.get(i) << " ";
      }
      return os;
    }

    // set
    void set(int32_t element, int32_t i, int32_t j=0, int32_t k=0, int32_t l=0) {
      SIDL_int__array_set4(d_array,i,j,k,l,element);
    }
  };

  template<> 
  class array<long> : public array_mixin<struct SIDL_long__array, int64_t, int64_t > {
  public:
    // default constructor
    array() : array_mixin< struct SIDL_long__array, int64_t, int64_t>() {}

    // constructor/casting operator
    array( SIDL_long__array* src ) : array_mixin< struct SIDL_long__array, int64_t, int64_t>(src) {}

    // create
    bool create(int32_t dimen, const int32_t lower[], const int32_t upper[]) {
      d_array = SIDL_long__array_create(dimen, lower, upper);
      return d_array != 0;
    }
    
    bool create(int32_t m, long *args) {
      int32_t lower = 0,upper = m-1;
      d_array = SIDL_long__array_create(1, &lower, &upper);
      if (!d_array) return false;
      int i; for (i=0; i<m; i++) this->set(args[i],i);
      return true;
    }

    // borrow
    bool borrow( int64_t * first_element, int32_t dimen,
      const int32_t lower[], const int32_t upper[], const int32_t stride[]) {
      d_array = SIDL_long__array_borrow(first_element, dimen, lower,
        upper, stride);
      return d_array != 0;
    }

    // destroy
    void destroy() {
      SIDL_long__array_destroy(d_array);
    }

    // get dimension
    int32_t dim() const { 
      return SIDL_long__array_dimen(d_array);
    }
    
    // get lower bound along particular dimension
    int32_t lower( int32_t d ) const {
      return SIDL_long__array_lower(d_array,d);
    } 
    
    // get upper bound along particular dimension
    int32_t upper( int32_t d ) const {
      return SIDL_long__array_upper(d_array,d);
    }
    
    // get stride along particular dimension
    // int32_t stride( int32_t d ) const {
    // 	 return SIDL_long__array_stride(d_array,d);
    // }

    // get
    int64_t get(int32_t i, int32_t j=0, int32_t k=0, int32_t l=0) {
      return SIDL_long__array_get4(d_array,i,j,k,l);
    }

    int64_t operator [](int32_t i) {return get(i);}

    friend std::ostream& operator <<(std::ostream &os,array<long> me) {
      for (int i=me.lower(0); i<me.upper(0)+1;i++) {
	os << me.get(i) << " ";
      }
      return os;
    }

    // set
    void set(int64_t element, int32_t i, int32_t j=0, int32_t k=0, int32_t l=0) {
      SIDL_long__array_set4(d_array,i,j,k,l,element);
    }
  };

  template<> 
  class array<int64_t> : public array_mixin<struct SIDL_long__array, int64_t, int64_t > {
  public:
    // default constructor
    array() : array_mixin< struct SIDL_long__array, int64_t, int64_t>() {}

    // constructor/casting operator
    array( SIDL_long__array* src ) : array_mixin< struct SIDL_long__array, int64_t, int64_t>(src) {}

    // create
    bool create(int32_t dimen, const int32_t lower[], const int32_t upper[]) {
      d_array = SIDL_long__array_create(dimen, lower, upper);
      return d_array != 0;
    }
    
    bool create(int32_t m, int64_t *args) {
      int32_t lower = 0,upper = m-1;
      d_array = SIDL_long__array_create(1, &lower, &upper);
      if (!d_array) return false;
      int i; for (i=0; i<m; i++) this->set(args[i],i);
      return true;
    }

    // borrow
    bool borrow( int64_t * first_element, int32_t dimen,
      const int32_t lower[], const int32_t upper[], const int32_t stride[]) {
      d_array = SIDL_long__array_borrow(first_element, dimen, lower,
        upper, stride);
      return d_array != 0;
    }

    // destroy
    void destroy() {
      SIDL_long__array_destroy(d_array);
    }

    // get dimension
    int32_t dim() const { 
      return SIDL_long__array_dimen(d_array);
    }
    
    // get lower bound along particular dimension
    int32_t lower( int32_t d ) const {
      return SIDL_long__array_lower(d_array,d);
    } 
    
    // get upper bound along particular dimension
    int32_t upper( int32_t d ) const {
      return SIDL_long__array_upper(d_array,d);
    }
    
    // get stride along particular dimension
    // int32_t stride( int32_t d ) const {
    // 	 return SIDL_long__array_stride(d_array,d);
    // }

    // get
    int64_t get(int32_t i, int32_t j=0, int32_t k=0, int32_t l=0) {
      return SIDL_long__array_get4(d_array,i,j,k,l);
    }

    int64_t operator [](int32_t i) {return get(i);}

    friend std::ostream& operator <<(std::ostream &os,array<int64_t> me) {
      for (int i=me.lower(0); i<me.upper(0)+1;i++) {
	os << me.get(i) << " ";
      }
      return os;
    }

    // set
    void set(int64_t element, int32_t i, int32_t j=0, int32_t k=0, int32_t l=0) {
      SIDL_long__array_set4(d_array,i,j,k,l,element);
    }
  };

  template<> 
  class array<float> : public array_mixin<struct SIDL_float__array, float, float > {
  public:
    // default constructor
    array() : array_mixin< struct SIDL_float__array, float, float>() {}

    // constructor/casting operator
    array( SIDL_float__array* src ) : array_mixin< struct SIDL_float__array, float, float>(src) {}

    // create
    bool create(int32_t dimen, const int32_t lower[], const int32_t upper[]) {
      d_array = SIDL_float__array_create(dimen, lower, upper);
      return d_array != 0;
    }
    
    bool create(int32_t m, float *args) {
      int32_t lower = 0,upper = m-1;
      d_array = SIDL_float__array_create(1, &lower, &upper);
      if (!d_array) return false;
      int i; for (i=0; i<m; i++) this->set(args[i],i);
      return true;
    }

    // borrow
    bool borrow( float * first_element, int32_t dimen,
      const int32_t lower[], const int32_t upper[], const int32_t stride[]) {
      d_array = SIDL_float__array_borrow(first_element, dimen, lower,
        upper, stride);
      return d_array != 0;
    }

    // destroy
    void destroy() {
      SIDL_float__array_destroy(d_array);
    }

    // get dimension
    int32_t dim() const { 
      return SIDL_float__array_dimen(d_array);
    }
    
    // get lower bound along particular dimension
    int32_t lower( int32_t d ) const {
      return SIDL_float__array_lower(d_array,d);
    } 
    
    // get upper bound along particular dimension
    int32_t upper( int32_t d ) const {
      return SIDL_float__array_upper(d_array,d);
    }
    
    // get stride along particular dimension
    // int32_t stride( int32_t d ) const {
    // 	 return SIDL_float__array_stride(d_array,d);
    // }

    // get
    float get(int32_t i, int32_t j=0, int32_t k=0, int32_t l=0) {
      return SIDL_float__array_get4(d_array,i,j,k,l);
    }

    float operator [](int32_t i) {return get(i);}

    friend std::ostream& operator <<(std::ostream &os,array<float> me) {
      for (int i=me.lower(0); i<me.upper(0)+1;i++) {
	os << me.get(i) << " ";
      }
      return os;
    }

    // set
    void set(float element, int32_t i, int32_t j=0, int32_t k=0, int32_t l=0) {
      SIDL_float__array_set4(d_array,i,j,k,l,element);
    }
  };

  template<> 
  class array<double> : public array_mixin<struct SIDL_double__array, double, double > {
  public:
    // default constructor
    array() : array_mixin< struct SIDL_double__array, double, double>() {}

    // constructor/casting operator
    array( SIDL_double__array* src) : array_mixin< struct SIDL_double__array, double, double>(src) {}

    // create
    bool create(int32_t dimen, const int32_t lower[], const int32_t upper[]) {
      d_array = SIDL_double__array_create(dimen, lower, upper);
      return d_array != 0;
    }
    
    bool create(int32_t m, double *args) {
      int32_t lower = 0,upper = m-1;
      d_array = SIDL_double__array_create(1, &lower, &upper);
      if (!d_array) return false;
      int i; for (i=0; i<m; i++) this->set(args[i],i);
      return true;
    }

    // borrow
    bool borrow( double * first_element, int32_t dimen,
      const int32_t lower[], const int32_t upper[], const int32_t stride[]) {
      d_array = SIDL_double__array_borrow(first_element, dimen, lower,
        upper, stride);
      return d_array != 0;
    }

    // destroy
    void destroy() {
      SIDL_double__array_destroy(d_array);
    }

    // get dimension
    int32_t dim() const { 
      return SIDL_double__array_dimen(d_array);
    }
    
    // get lower bound along particular dimension
    int32_t lower( int32_t d ) const {
      return SIDL_double__array_lower(d_array,d);
    } 
    
    // get upper bound along particular dimension
    int32_t upper( int32_t d ) const {
      return SIDL_double__array_upper(d_array,d);
    }
    
    // get stride along particular dimension
    // int32_t stride( int32_t d ) const {
    // 	 return SIDL_double__array_stride(d_array,d);
    // }

    // get
    double get(int32_t i, int32_t j=0, int32_t k=0, int32_t l=0) {
      return SIDL_double__array_get4(d_array,i,j,k,l);
    }

    double operator [](int32_t i) {return get(i);}

    friend std::ostream& operator <<(std::ostream &os,array<double> me) {
      for (int i=me.lower(0); i<me.upper(0)+1;i++) {
	os << me.get(i) << " ";
      }
      return os;
    }

    // set
    void set(double element, int32_t i, int32_t j=0, int32_t k=0, int32_t l=0) {
      SIDL_double__array_set4(d_array,i,j,k,l,element);
    }
  };

  template<> 
  class array<fcomplex> : public array_mixin<struct SIDL_fcomplex__array, long, fcomplex > {
  public:
    // default constructor
    array() : array_mixin< struct SIDL_fcomplex__array, long, fcomplex >() {}

    // constructor/casting operator
    array( SIDL_fcomplex__array* src ) : array_mixin< struct SIDL_fcomplex__array, long, fcomplex >(src) {}

    // create
    bool create(int32_t dimen, const int32_t lower[], const int32_t upper[]) {
      d_array = SIDL_fcomplex__array_create(dimen, lower, upper);
      return d_array != 0;
    }
    
    bool create(int32_t m, fcomplex *args) {
      int32_t lower = 0,upper = m-1;
      d_array = SIDL_fcomplex__array_create(1, &lower, &upper);
      if (!d_array) return false;
      int i; for (i=0; i<m; i++) this->set(args[i],i);
      return true;
    }

    // borrow
    bool borrow( struct SIDL_fcomplex * first_element, int32_t dimen,
      const int32_t lower[], const int32_t upper[], const int32_t stride[]) {
      d_array = SIDL_fcomplex__array_borrow(first_element, dimen, lower,
        upper, stride);
      return d_array != 0;
    }

    // destroy
    void destroy() {
      SIDL_fcomplex__array_destroy(d_array);
    }

    // get dimension
    int32_t dim() const { 
      return SIDL_fcomplex__array_dimen(d_array);
    }
    
    // get lower bound along particular dimension
    int32_t lower( int32_t d ) const {
      return SIDL_fcomplex__array_lower(d_array,d);
    } 
    
    // get upper bound along particular dimension
    int32_t upper( int32_t d ) const {
      return SIDL_fcomplex__array_upper(d_array,d);
    }
    
    // get stride along particular dimension
    // int32_t stride( int32_t d ) const {
    // 	 return SIDL_fcomplex__array_stride(d_array,d);
    // }

    // get
    fcomplex get(int32_t i, int32_t j=0, int32_t k=0, int32_t l=0) {
      SIDL_fcomplex _local_result = 
	SIDL_fcomplex__array_get4(d_array,i,j,k,l);
      return reinterpret_cast<fcomplex&>(_local_result);
    }

    fcomplex operator [](int32_t i) {return get(i);}

    friend std::ostream& operator <<(std::ostream &os,array<fcomplex> me) {
      for (int i=me.lower(0); i<me.upper(0)+1;i++) {
	os << me.get(i) << " ";
      }
      return os;
    }

    // set
    void set(fcomplex element, int32_t i, int32_t j=0, int32_t k=0, int32_t l=0) {
      SIDL_fcomplex _local_element = { element.real(), element.imag() };
      SIDL_fcomplex__array_set4(d_array,i,j,k,l,_local_element);
    }
  };

  template<> 
  class array<dcomplex> : public array_mixin<struct SIDL_dcomplex__array, long, dcomplex > {
  public:
    // default constructor
    array() : array_mixin< struct SIDL_dcomplex__array, long, dcomplex >() {}
    // constructor/casting operator
    array( SIDL_dcomplex__array* src ) : array_mixin< struct SIDL_dcomplex__array, long, dcomplex >(src) {}

    // create
    bool create(int32_t dimen, const int32_t lower[], const int32_t upper[]) {
      d_array = SIDL_dcomplex__array_create(dimen, lower, upper);
      return d_array != 0;
    }
    
    bool create(int32_t m, dcomplex *args) {
      int32_t lower = 0,upper = m-1;
      d_array = SIDL_dcomplex__array_create(1, &lower, &upper);
      if (!d_array) return false;
      int i; for (i=0; i<m; i++) this->set(args[i],i);
      return true;
    }

    // borrow
    bool borrow( struct SIDL_dcomplex * first_element, int32_t dimen,
      const int32_t lower[], const int32_t upper[], const int32_t stride[]) {
      d_array = SIDL_dcomplex__array_borrow(first_element, dimen, lower,
        upper, stride);
      return d_array != 0;
    }

    // destroy
    void destroy() {
      SIDL_dcomplex__array_destroy(d_array);
    }

    // get dimension
    int32_t dim() const { 
      return SIDL_dcomplex__array_dimen(d_array);
    }
    
    // get lower bound along particular dimension
    int32_t lower( int32_t d ) const {
      return SIDL_dcomplex__array_lower(d_array,d);
    } 
    
    // get upper bound along particular dimension
    int32_t upper( int32_t d ) const {
      return SIDL_dcomplex__array_upper(d_array,d);
    }
    
    // get stride along particular dimension
    // int32_t stride( int32_t d ) const {
    // 	 return SIDL_dcomplex__array_stride(d_array,d);
    // }

    // get
    dcomplex get(int32_t i, int32_t j=0, int32_t k=0, int32_t l=0) {
      SIDL_dcomplex  _local_result = 
	SIDL_dcomplex__array_get4(d_array,i,j,k,l);
      return reinterpret_cast<dcomplex&>(_local_result);
    }

    dcomplex operator [](int32_t i) {return get(i);}

    friend std::ostream& operator <<(std::ostream &os,array<dcomplex> me) {
      for (int i=me.lower(0); i<me.upper(0)+1;i++) {
	os << me.get(i) << " ";
      }
      return os;
    }

    // set
    void set(dcomplex element, int32_t i, int32_t j=0, int32_t k=0, int32_t l=0) {
      SIDL_dcomplex _local_element = { element.real(), element.imag() };
      SIDL_dcomplex__array_set4(d_array,i,j,k,l,_local_element);
    }
  };

  template<> 
  class array<opaque> : public array_mixin<struct SIDL_opaque__array, void*, void*> { 
  public:
    // default constructor
    array() : array_mixin< struct SIDL_opaque__array, void*, void*>() {}

    // constructor/casting operator
    array( SIDL_opaque__array* src ) : array_mixin< struct SIDL_opaque__array, void*, void*>(src) {}

    // create
    bool create(int32_t dimen, const int32_t lower[], const int32_t upper[]) {
      d_array = SIDL_opaque__array_create(dimen, lower, upper);
      return d_array != 0;
    }
    
    bool create(int32_t m, void **args) {
      int32_t lower = 0,upper = m-1;
      d_array = SIDL_opaque__array_create(1, &lower, &upper);
      if (!d_array) return false;
      int i; for (i=0; i<m; i++) this->set(args[i],i);
      return true;
    }

    // borrow
    bool borrow( void ** first_element, int32_t dimen,
      const int32_t lower[], const int32_t upper[], const int32_t stride[]) {
      d_array = SIDL_opaque__array_borrow(first_element, dimen, lower,
	upper, stride);
      return d_array != 0;
    }

    // destroy
    void destroy() {
      SIDL_opaque__array_destroy(d_array);
    }

    // get dimension
    int32_t dim() const { 
      return SIDL_opaque__array_dimen(d_array);
    }
    
    // get lower bound along particular dimension
    int32_t lower( int32_t d ) const {
      return SIDL_opaque__array_lower(d_array,d);
    } 
    
    // get upper bound along particular dimension
    int32_t upper( int32_t d ) const {
      return SIDL_opaque__array_upper(d_array,d);
    }
    
    // get stride along particular dimension
    // int32_t stride( int32_t d ) const {
    // 	 return SIDL_opaque__array_stride(d_array,d);
    // }

    // get
    void* get(int32_t i, int32_t j=0, int32_t k=0, int32_t l=0) {
      return SIDL_opaque__array_get4(d_array,i,j,k,l);
    }

    void* operator [](int32_t i) {return get(i);}

    // set
    void set(void* element, int32_t i, int32_t j=0, int32_t k=0, int32_t l=0) {
      SIDL_opaque__array_set4(d_array,i,j,k,l,element);
    }
  };

  template<> 
  class array<string> : public array_mixin<struct SIDL_string__array, char*, string > {
  public:
    // default constructor
    array() : array_mixin< struct SIDL_string__array, char*, string>() {}

    // constructor/casting operator
    array( SIDL_string__array* src) : array_mixin< struct SIDL_string__array, char*, string>(src) {}

    // create
    bool create(int32_t dimen, const int32_t lower[], const int32_t upper[]) {
      d_array = SIDL_string__array_create(dimen, lower, upper);
      return d_array != 0;
    }
    
    bool create(int32_t m, char **args) {
      int32_t lower = 0,upper = m-1;
      d_array = SIDL_string__array_create(1, &lower, &upper);
      if (!d_array) return false;
      int i; for (i=0; i<m; i++) this->set(args[i],i);
      return true;
    }

    // borrow
    bool borrow( char ** first_element, int32_t dimen,
      const int32_t lower[], const int32_t upper[], const int32_t stride[]) {
      d_array = SIDL_string__array_borrow(first_element, dimen, lower,
        upper, stride);
      return d_array != 0;
    }

    // destroy
    void destroy() {
      SIDL_string__array_destroy(d_array);
    }

    // get dimension
    int32_t dim() const { 
      return SIDL_string__array_dimen(d_array);
    }
    
    // get lower bound along particular dimension
    int32_t lower( int32_t d ) const {
      return SIDL_string__array_lower(d_array,d);
    } 
    
    // get upper bound along particular dimension
    int32_t upper( int32_t d ) const {
      return SIDL_string__array_upper(d_array,d);
    }
    
    // get stride along particular dimension
    // int32_t stride( int32_t d ) const {
    // 	 return SIDL_string__array_stride(d_array,d);
    // }

    // get
    string get(int32_t i, int32_t j=0, int32_t k=0, int32_t l=0) {
      // special case: 
      // must make a local copy, since the return value of 
      // SIDL_string__array_get4 must be free'd
      char *tmp = SIDL_string__array_get4(d_array,i,j,k,l);
      string s(tmp);
      free(tmp);
      return s;
    }

    string operator [](int32_t i) {return get(i);}

    friend std::ostream& operator <<(std::ostream &os,array<string> me) {
      for (int i=me.lower(0); i<me.upper(0)+1;i++) {
	os << me.get(i) << " ";
      }
      return os;
    }

    // set
    void set(string element, int32_t i, int32_t j=0, int32_t k=0, int32_t l=0) {
      SIDL_string__array_set4(d_array,i,j,k,l,element.c_str());
    }
  };
}

#endif /* included_SIDL_cxx_hh */
